use <mockup/bearing.scad>
use <mockup/stepper.scad>
use <pillow_block.scad>

// import pre-rendered objects instead of calling these as libraries, for faster assembly preview
//use <openscad-openbuilds/linear_rails/vslot.scad>;
//use <gears.scad> 

// Arrangement of things along the shaft:
//   motor-driveshaft coupler -> drive gear -> pillow block -> bearing -> washer -> stop collar -> 
//    empty middle section -> 
//   drive gear 2 -> stop collar2 -> washer -> bearing -> pillow block 2


separation = 44;
// (2.5x extrusion unit) - nut_block_th/2  =>  50 - 12/2  =>  44

gear1_teeth = 16;
gear1_height = 15;

gear2_teeth = 32;
gear2_height = 15;

iterations = 120;

// TODO differentiate coupler settings for both motor-shaft and leadscrew-gearshaft
motor_coupler_d = 19.75;
motor_coupler_h = 25;

motor_length = 47.5;
motor_shaft_length = 24.5;
// distance above extrusion surface, based on mount plate position
motor_center_offset = 22.55;
motor_mount_th = 3;

washerID = 8.4;
washerOD = 15.7;
washerTh = 1.5;

// Adjust drive gear rotation by a fraction of a tooth, only matters if you want the gears to look properly meshed in assembly mockup.
teeth_mesh_correction = 0.17; // [0:0.01:1]

bearingID = 8;
bearingOD = 22;
bearingTh = 7;
bearingTxt = "608-2RS";


/* [Hidden] */
$fs = 0.5;
$fa = 0.1;
COLOR_STEEL = [0.75,0.75,0.75];
COLOR_AL = [0.77, 0.77, 0.8];

// derived values

// Height from bottom of pillow block to bottom of bearing
pillow_block_bearing_height = motor_center_offset - bearingOD/2;
m = separation / ((gear1_teeth+gear2_teeth)/2);


mount_w=39.50;
mount_h=82.00;

coupler_gear_gap = 0;
coupler_motor_insert_length = 18.5;
coupler_shaft_insert_length = 5;

motor_coupler_gap = motor_shaft_length - coupler_motor_insert_length + 2;
//echo(motor_coupler_gap);
motor_offset = motor_mount_th + motor_coupler_gap + motor_coupler_h + coupler_gear_gap + gear1_height/2;

// distance(along Y) between edge of extrusion and output gear axis
gear_mount_edge_y = separation - 30;
// distance(along Z) between edge of extrusion and output gear axis
gear_mount_edge_z = motor_center_offset;
//echo(gear_mount_edge_y, gear_mount_edge_z);


// printer frame top extrusion piece
translate([-40,-30-(separation-30),-10-motor_center_offset]) 
  rotate([90,0,90]) 
   color(COLOR_AL) import("mockup/vslot_20x60x450.stl"); //vslot20x60(450);

// drive gear
color("red") translate([0,-separation,0]) rotate([0,90,0]) 
  rotate(360*$t + (teeth_mesh_correction * 360/gear1_teeth) ) 
    import("gear1.stl"); //chamfered_gear(m, gear1_teeth, gear1_height, 8, 2, 1, iterations);

// output gear mount
translate([0,-gear_mount_edge_y,-gear_mount_edge_z-20]) rotate([-90,0,0]) rotate(180) 
  import("gear_mount.stl");

// output gear
color("red") rotate(360*gear1_teeth/gear2_teeth*$t)
  import("gear2.stl"); //chamfered_gear(m, gear2_teeth, gear2_height, 8, 2, 1, iterations);

// output gear washers
translate([0,0,gear2_height/2])
  washer(washerID,washerOD,washerTh);
translate([0,0,-gear2_height/2-washerTh])
  washer(washerID,washerOD,washerTh);

// output shaft
color(COLOR_STEEL) translate([0,0,-42]) cylinder(d=8,h=60);

// gear lower bearing + washer
translate([0,0,-gear2_height/2-washerTh]) {
  translate([0,0,-bearingTh]) bearing(bearingID, bearingOD, bearingTh, bearingTxt);
  washer();
}

// gear upper bearing + washer
translate([0,0,gear2_height/2]) {
  translate([0,0,washerTh]) bearing(bearingID, bearingOD, bearingTh, bearingTxt);
  washer();
}

// 8mmx8mm coupler
translate([0,0,-58]) 
  color("red") import("coupler_8x8.stl");



translate([0,-separation,0]) rotate([0,90,0])
{
  // Stepper Motor
  translate([0,0,-motor_length-motor_offset])
    NEMA17(motor_length);

  // motor mount plate
    // FIXME 1.25mm fudge factor?
  rotate(90) translate([-mount_w/2,-mount_h+motor_center_offset-1.25,-motor_offset]) 
    color([0.2,0.2,0.2]) import("mockup/motor_mount_plate.stl");

  translate([0,0,-motor_offset+motor_coupler_gap]) {
    // shaft coupler
    %cylinder(d=motor_coupler_d, h=motor_coupler_h);

    // driveshaft 
    color(COLOR_STEEL) translate([0,0,motor_coupler_h-coupler_shaft_insert_length]) cylinder(d=8, h=400);
  }
}

// Pillow block
translate([gear1_height/2+22,-separation,-pillow_block_bearing_height-bearingOD/2+0.01]) rotate(90) 
  pillow_block(8,22,7,"608-2RS",pillow_block_bearing_height,0.2,3,5,5,1,5,40,5,10,0.4,2,true);

// washer + stop collar
translate([gear1_height/2+22,-separation,]) rotate([0,90,0]) {
  washer(washerID,washerOD,washerTh);
  rotate(360*$t) translate([0,0,washerTh]) color("red") import("stop_collar.stl");
}


module washer(washerID,washerOD,washerTh) {
  color(COLOR_STEEL) linear_extrude(height=washerTh) difference() {
    circle(d=washerOD);
    circle(d=washerID);
  } 
}