$fa=0.1;
$fs=0.5;
bearing(8,22,7,"608-2RS");
//sealedBearing(8, 22, 7, "608-2RS    OpenSCAD", 2, txtRepeat=1, $fs=0.1, $fa=0.1);
//demo8mm();
//pileOfBearings();

module demo8mm() {
  // 8mm ID bearing comparison
  translate([-56, 0]) bearing(8, 28, 9, "638-2RS", 2.0, $fs=0.1, $fa=0.1);
  translate([-28, 0]) bearing(8, 24, 8, "628-2RS", 2.0, $fs=0.1, $fa=0.1);
  translate([  0, 0]) bearing(8, 22, 7, "608-2RS", 2.0, $fs=0.1, $fa=0.1);
  translate([ 28, 0]) bearing(8, 19, 6, "698-2RS", 2.0, $fs=0.1, $fa=0.1);
  translate([ 56, 0]) bearing(8, 16, 5, "688-2RS", 1.6, $fs=0.1, $fa=0.1);
}

module pileOfBearings() {
  for(i=[0:100])
    translate(rands(-100,100,3))
      bearing(8, 22, 7, "608-2RS", 2, $fs=0.1, $fa=0.1);
}


// for now only one type, maybe extend to other types later, such as shielded
module bearing(d, D, th, 
    txt, txtSize, txtRepeat=2, spacing=0.85, font,
    steelColor = [0.75,0.75,0.78], sealColor = [0.15,0.15,0.15],
    sealOffset = 0.25,
    chamfer = 0.2,
    textHeight = 0.15) {
  txtSize = txtSize == undef ? (D - d)/7 : txtSize;
  sealedBearing(d,D,th,txt,txtSize,txtRepeat,spacing,font,steelColor,sealColor,sealOffset,chamfer,textHeight);
}


// A quality looking bearing for mockups in OpenSCAD
// Optional curved text on front bearing seal
// Only d, D, th are required for basic usage
module sealedBearing(d, D, th, 
  txt, txtSize, txtRepeat, spacing, font, 
  steelColor, sealColor, sealOffset, chamfer, 
  textHeight) {
  
  r = d/2;
  R = D/2;
  x = (R-r)/4; // race thickness
  txtPadding = x-txtSize/2; // amount of space above/below text on seal

  color(steelColor) 
  rotate_extrude() {
    // outer race
    polygon([ [R-x, 0], [R-chamfer, 0], [R, chamfer], [R,th-chamfer], [R-chamfer,th], [R-x,th] ]);

    // inner race
    polygon([ [r, chamfer], [r+chamfer, 0], [r+x,0], [r+x,th], [r+chamfer,th], [r,th-chamfer] ]);
  }

  // seal
  color(sealColor) rotate_extrude()
    translate([r+x/2,sealOffset]) square([(R-r)*0.75, th-sealOffset*2]);

  // seal text
  color(1.2*sealColor) // slightly brighter so letters visually stand out
    for (i = [0:txtRepeat-1])
    rotate([0,0,(360/txtRepeat)*i]) translate([0,0,th-sealOffset-textHeight]) 
      linear_extrude(2*textHeight) 
        curvedText(txt, r=r+x+txtPadding, size=txtSize, spacing=spacing, font=font);
  
}


module curvedText(txt, r=10, size=10, spacing=1, valign="baseline", font) {
  a = 180 * size * spacing / (PI * r);
  for (i = [0:len(txt)-1])
    rotate([0,0,-(i+0.5)*a]) 
    translate([0,r]) text(txt[i], size=size, halign="center", valign=valign, $fn=32, font=font);
}
