use <BOLTS.scad>
use <bearing.scad>
use <RetainingRings.scad>

stepperDemo();
//translate([50,50]) bearing(8,22,7);

module stepperDemo() {
  $fs=0.1;
  $fa=0.1;

  // 17HS15-1684S-PG5
  translate([0,0,-27.15]) {
    translate([0,0,-47]) NEMA17(47);
    NEMA17PlanetaryGearbox(27.15);
  }
  // http://www.automationtechnologiesinc.com/wp-content/uploads/2013/07/3Dprintermotor.jpg
  // http://www.automationtechnologiesinc.com/wp-content/uploads/downloads/2013/07/KL17H248-15-4A.pdf
  // KL17H248-15-4A 
  translate([50,0,-47.5]) NEMA17_2(47.5);
    
  // http://www.omc-stepperonline.com/images/11HS12-0674D-PG27.jpg
  translate([-50,0,-35]) {
    translate([0,0,-31.6]) NEMA11(31.5);
    NEMA11PlanetaryGearbox(35);
  }
}

// chamfered version
module NEMA17(length) {
  stepperMotorChamfered(
    length=length, caseWidth=42.3, caseFlatsWidth=53.57, caseInnerFlatsWidth=50.25,
    boltPatternD=31*sqrt(2), boltD=3, boltDepth=4.5,
    topThickness=10, topcutThickness=7.5, topProtrude=2, topProtrudeD=22, 
    topRecessOD=32.5, topRecessID=30.85, topRecessDepth=0.6,
    bottomThickness=11, bottomcutThickness=9,
    shaftD=5, shaftLength=24.5, flatD=4.5, flatLength=21, rearShaftLength=-3, shaftChamfer=0.5
  );
}

// rounded version
module NEMA17_2(length) {
  stepperMotorRounded(
    length=length, caseWidth=42.3, caseRadius=25,
    boltPatternD=31*sqrt(2), boltD=3, boltDepth=4.5,
    topThickness=8, topProtrude=2, topProtrudeD=22, topHoleD=7.5,
    bottomThickness=9, bottomHoleD=7,
    topRecessOD=32.5, topRecessID=30.85, topRecessDepth=0.6,
    shaftD=5, shaftLength=24.5, flatD=4.5, flatLength=21, rearShaftLength=-3, shaftChamfer=0.5
  );
}

module NEMA11(length) {
  stepperMotorRounded(
    length=length, caseWidth=28.194, caseRadius=2,
    boltPatternD=32.54, boltD=2.5, boltDepth=4,
    topThickness=7, topProtrude=1.5, topProtrudeD=22, topHoleD=6,
    bottomThickness=7, bottomHoleD = 6,
    topRecessOD=26, topRecessID=24, topRecessDepth=0.6,
    shaftD=5, shaftLength=20, flatD=4.5, flatLength=0, rearShaftLength=10, shaftChamfer=0.5
  );
}

module NEMA11PlanetaryGearbox(caseLength) {
  planetaryGearbox(
    flangeWidth=28.1, flangeFlatWidth=37.95, flangeThickness=4.8,
    flangeBoltPatternD=32.69, flangeCounterboreD=0, flangeCounterboreDepth=0, flangeBolt="M2.5",
    caseD=28, caseLength=caseLength,
    mountPlateThickness=7, hasMountPlateBolts=false,
    boltPatternD=22, boltD=3, boltDepth=6,
    lipD=18, lipThickness=2, 
    bearingID=6, bearingOD=15, bearingTh=5, bearingText="696RS", bearingTextSize=1.25,
    shaftD=6, shaftLength=18, flatD=5, flatLength=12, shaftChamfer=0.75
  );
}

module NEMA17PlanetaryGearbox(caseLength) {
  planetaryGearbox(
    flangeWidth=40, flangeFlatWidth=52, flangeThickness=4.8, 
    flangeBoltPatternD=1.725*25.4, flangeCounterboreD=6.25, flangeCounterboreDepth=3,
    caseD=36, caseLength=caseLength,
    mountPlateThickness=7, hasMountPlateBolts=true, counterboreDepth=3.5, counterboreD=5.5,
    boltPatternD=28, boltD=3, boltDepth=6,
    lipD=22, lipThickness=2, 
    bearingID=8, bearingOD=19, bearingTh=6, bearingText="698RS", bearingTextSize=1.5,
    shaftD=8, shaftLength=20, flatD=7, flatLength=15, shaftChamfer=0.75
  );
}

module planetaryGearbox(
    flangeWidth, flangeFlatWidth, flangeThickness, 
    flangeBoltPatternD, flangeCounterboreD, flangeCounterboreDepth, flangeBolt="M3",
    caseD, caseLength,
    mountPlateThickness, hasMountPlateBolts=true, counterboreDepth, counterboreD,
    boltPatternD, boltD, boltDepth,
    lipD, lipThickness, 
    bearingID, bearingOD, bearingTh, bearingText, bearingTextSize,
    shaftD, shaftLength, flatD, flatLength, shaftChamfer) {

  alColor = [0.9,0.9,0.9];
  steelColor = [0.75,0.75,0.75];
  tanColor = [0.8,0.8,0.7];
  blackColor = [0.15,0.15,0.15];
  
  translate([0,0,caseLength+lipThickness-bearingTh-0.02]) 
    bearing(bearingID,bearingOD,bearingTh,bearingText,bearingTextSize);

      
  difference() {
    union() {
      // flange
      color(alColor) linear_extrude(flangeThickness) intersection() {
        square(flangeWidth, center=true);
        rotate([0,0,45]) square(flangeFlatWidth, center=true);
      }

      color(alColor) {
        //mount plate
        difference() {
          translate([0,0,caseLength-mountPlateThickness]) cylinder(d=caseD, h=mountPlateThickness);
          translate([0,0,caseLength-mountPlateThickness]) rotate([180,0]) cylindricalChamferOD(caseD,0.1);
        }
        // lip
        translate([0,0,caseLength-1]) cylinder(d=lipD, h=lipThickness+1);
      }

      // case
      color(tanColor) cylinder(d=caseD, h=caseLength-mountPlateThickness);
    }
    
    // flange counterbores
    color(alColor) for (i = [0:3]) rotate([0,0,45+90*i])
      translate([flangeBoltPatternD/2,0,flangeThickness-flangeCounterboreDepth]) cylinder(d=flangeCounterboreD, h=flangeCounterboreDepth+1);  
    
    if (hasMountPlateBolts)
    // mount plate to case, bolt cutouts
    color(alColor) translate([0,0,caseLength-counterboreDepth]) linear_extrude(counterboreDepth+1, convexity=3) 
      for (i = [0:3]) rotate([0,0,90*i]) translate([boltPatternD/2,0]) {
        circle(d=counterboreD); 
        translate([0,-counterboreD/2]) square([counterboreD*2,counterboreD]);
      }
    
    // mounting screw holes
    color(alColor) for (i = [0:3]) rotate([0,0,45+90*i])
      translate([boltPatternD/2,0,caseLength-boltDepth]) cylinder(d=boltD, h=boltDepth+1);  
    
    // bearing hole
    color(alColor) {
      translate([0,0,caseLength+lipThickness-bearingTh]) cylinder(d=bearingOD, h=bearingTh+1);
      translate([0,0,caseLength+lipThickness]) cylindricalChamferID(bearingOD,0.1);
    }

  }            

  // output shaft
  color(blackColor)
  translate([0,0,caseLength]) difference() {
    cylinder(d=shaftD, h=shaftLength);
    translate([-shaftD/2,shaftD/2-(shaftD-flatD),shaftLength-flatLength]) 
      cube([shaftD, (shaftD-flatD)*2, flatLength+1]);
    translate([0,0,shaftLength])
      cylindricalChamferOD(shaftD, shaftChamfer);
  }
  
  // snap ring
  translate([0,0,caseLength+lipThickness])
  ANSI_3AM1(shaftD); //external_retaining_ring(8,7.15,0.6,1,2.2,1,0.65,7.5,0.7,0.25,0.8);  
  
  // BOLTS connector functionality not working https://github.com/jreinhardt/BOLTS/issues/158
  //part_cs = new_cs(origin = , axes = [[0,0,1],[0,-1,0]]);
  //bolt_cs = DINENISO4762_conn("M2.5", "top");
  //align(bolt_cs, part_cs) 
  
  if (hasMountPlateBolts)
  // mount plate to case bolts
  for (i = [0:3]) rotate([0,0,i*90])
  color(blackColor) 
    translate([boltPatternD/2,0,caseLength-counterboreDepth]) 
      rotate([180,0]) DINENISO4762(key="M2.5", l=8);
  
  // flange bolts
  for (i = [0:3]) rotate([0,0,45+i*90]) color(steelColor) 
    translate([flangeBoltPatternD/2,0,flangeThickness-flangeCounterboreDepth]) 
      rotate([180,0]) DINENISO4762(key=flangeBolt, l=8);
  
}


// Stepper motor with chamfered corners on caps. Caps extend past rest of body
module stepperMotorChamfered(
    length, caseWidth, caseFlatsWidth, caseInnerFlatsWidth,
    boltPatternD, boltD, boltDepth,
    topThickness, topcutThickness, topProtrude, topProtrudeD, 
    topRecessOD, topRecessID, topRecessDepth,
    bottomThickness, bottomcutThickness) {

  color1 = [0.05,0.05,0.05];
  color2 = [0.15,0.15,0.15];
  alColor = [0.9,0.9,0.9];
  steelColor = [0.75,0.75,0.75];
  
  difference() {
    // end caps
     color(color2) linear_extrude(length+0.1) intersection() {
      square(caseWidth, center=true);
      rotate([0,0,45]) square(caseFlatsWidth, center=true);
    }
       
    // top face
    color(color2) translate([0,0,length-topRecessDepth]) difference() {
      cylinder(d=topRecessOD, h=topProtrude+1);
      rotate([180,0]) cylindricalChamferOD(topRecessOD, topRecessDepth);
      translate([0,0,-0.5]) cylinder(d=topProtrudeD-0.2,h=topProtrude*2);
    }
    
    // cut bare top face
    color(alColor) translate([0,0,length-0.02])
      linear_extrude(topProtrude+0.1, convexity=3) difference() {
        square(caseWidth+1,center=true);
        circle(d=(topProtrudeD+topRecessOD)/2, $fn=12);
      }
    
    // top bolt holes  
    color(alColor) translate([0,0,length-topThickness+1]) for (i=[0:3]) rotate(45+i*90) translate([boltPatternD/2,0]) cylinder(d=boltD, h=topThickness);

      
    // center cutout
    color(alColor) {
      translate([-caseWidth/2-1,-caseWidth/2-1,bottomThickness]) 
        cube([caseWidth+2,caseWidth+2,length-topThickness-bottomThickness]);
      translate([0,0,bottomcutThickness]) cylinder(d=caseInnerFlatsWidth+0.25,h=length-bottomcutThickness-topcutThickness);
    }
  }
  
  // top face protrusion
  difference() {
    color(alColor) translate([0,0,length-topProtrude]) cylinder(d=topProtrudeD+0.1, h=topProtrude*2);
    color(color2) translate([0,0,length+topProtrude]) cube([topProtrudeD+1,topProtrudeD+1,1], center=true);
  }

  // center section
  color(color1) translate([0,0,bottomcutThickness]) 
    linear_extrude(length-topcutThickness-bottomcutThickness) intersection() {
      square(caseWidth,center=true);
      circle(d=caseInnerFlatsWidth);
    }
    
  // motor shaft
  color(steelColor)
  translate([0,0,-rearShaftLength]) difference() {
    cylinder(d=shaftD, h=shaftLength+rearShaftLength+length);
    translate([-shaftD/2,shaftD/2-(shaftD-flatD),shaftLength+rearShaftLength+length-flatLength]) 
      cube([shaftD, (shaftD-flatD)*2, flatLength+1]); // shaft flat
    translate([0,0,shaftLength+rearShaftLength+length]) 
      cylindricalChamferOD(shaftD, shaftChamfer);// top chamfer
    rotate([180,0])
      cylindricalChamferOD(shaftD, shaftChamfer); // bottom chamfer
  }

  // bottom hole 6.8
  // top hole 8
  // 625 5x16x5 shielded
}

// Stepper motor with rounded corners.  Caps have consistent profile with body
module stepperMotorRounded(
    length, caseWidth, caseRadius,
    boltPatternD, boltD, boltDepth,
    topThickness, topProtrude, topProtrudeD, topHoleD,
    bottomThickness, bottomHoleD, 
    topRecessOD, topRecessID, topRecessDepth,
    shaftD, shaftLength, flatD, flatLength, rearShaftLength, shaftChamfer
  ) {

  color1 = [0.05,0.05,0.05];
  color2 = [0.15,0.15,0.15];
  alColor = [0.9,0.9,0.9];
  steelColor = [0.75,0.75,0.75];
  
  difference() {
    union() {
      color(color2) linear_extrude(bottomThickness, convexity=3) 
        difference() {
          roundedStepperProfile(caseWidth, caseRadius);
          circle(d=bottomHoleD);
        }
      color(color1) translate([0,0,bottomThickness]) linear_extrude(length-bottomThickness-topThickness) 
        roundedStepperProfile(caseWidth, caseRadius);
      color(color2) translate([0,0,length-topThickness]) linear_extrude(topThickness+0.1, convexity=3) 
        difference() {
          roundedStepperProfile(caseWidth, caseRadius);
          circle(d=topHoleD);
        }
    }
    
    // top face
    color(color2) translate([0,0,length-topRecessDepth]) difference() {
      cylinder(d=topRecessOD, h=topProtrude+1);
      rotate([180,0]) cylindricalChamferOD(topRecessOD, topRecessDepth);
      translate([0,0,-0.5]) cylinder(d=topProtrudeD-0.2,h=topProtrude*2);
    }
    
    // cut bare top face
    color(alColor) translate([0,0,length-0.02])
      linear_extrude(topProtrude+0.1, convexity=3) difference() {
        square(caseWidth+1,center=true);
        circle(d=(topProtrudeD+topRecessOD)/2, $fn=12);
      }
    
    // top bolt holes  
    color(alColor) translate([0,0,length-topThickness+1]) for (i=[0:3]) rotate(45+i*90) translate([boltPatternD/2,0]) cylinder(d=boltD, h=topThickness);

  }
  
  // top face protrusion
  translate([0,0,length]) difference() {
    color(alColor) translate([0,0,-topRecessDepth-0.1]) cylinder(d=topProtrudeD+0.1, h=topProtrude+topRecessDepth+0.1);
    color(color2) translate([0,0,topProtrude]) cube([topProtrudeD+1,topProtrudeD+1,1], center=true);
    color(color2) cylinder(d=topHoleD,h=(topProtrude+topRecessDepth)*2,center=true);
  }
  
  // motor shaft
  color(steelColor)
  translate([0,0,-rearShaftLength]) difference() {
    cylinder(d=shaftD, h=shaftLength+rearShaftLength+length);
    translate([-shaftD/2,shaftD/2-(shaftD-flatD),shaftLength+rearShaftLength+length-flatLength]) 
      cube([shaftD, (shaftD-flatD)*2, flatLength+1]); // shaft flat
    translate([0,0,shaftLength+rearShaftLength+length]) 
      cylindricalChamferOD(shaftD, shaftChamfer);// top chamfer
    rotate([180,0])
      cylindricalChamferOD(shaftD, shaftChamfer); // bottom chamfer
  }
  
}

module roundedStepperProfile(caseWidth, caseRadius) {
  if (caseRadius == caseWidth/2) {
    circle(r=caseRadius);
  } else if (caseRadius < caseWidth/2) {
    hull() for(i=[0:1],j=[0:1]) mirror([i,0]) mirror([0,j])
      translate([caseWidth/2-caseRadius,caseWidth/2-caseRadius]) circle(r=caseRadius);
  } else { // caseRadius > caseWidth/2
    intersection() {
      square(caseWidth, center=true);
      circle(r=caseRadius);
    }
  }
}

module cylindricalChamferOD(d, chamfer) {
  rotate_extrude() polygon([[d/2-chamfer,0],[d/2-chamfer,chamfer],[d/2+chamfer,chamfer],[d/2+chamfer,-chamfer],[d/2,-chamfer]]);
}

// generate a shape to be subtracted from an object to generate an innner chamfer
module cylindricalChamferID(d, chamfer) {
  rotate_extrude() polygon([[d/2+chamfer,0],[d/2+chamfer,chamfer],[d/2-chamfer,chamfer],[d/2-chamfer,-chamfer],[d/2,-chamfer]]);
}
